
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>

#include "sslfindkey.h"

static byte_sequence*
ssl_read_sequence(mem_access* memory){
	uint8_t byte = memory->read();
	uint16_t datalen = 0;
	if(byte != 0x02)
		return NULL;
	byte = memory->read();
	if(byte == 0x81)
		datalen |= memory->read();
	else if(byte == 0x82){
		datalen = memory->read()<<8;
		datalen |= memory->read();
	}
	else
		datalen = byte;
	byte_sequence* sequence = malloc(sizeof(sequence));
	if(!sequence)
		return NULL;
	sequence->len = datalen;
	sequence->data = malloc(sequence->len);
	if(!sequence->data)
		return NULL;

	int i;
	for(i=0;i<sequence->len;++i)
		sequence->data[i] = memory->read();
	if(memory->error()){
		free(sequence->data);
		free(sequence);
		return NULL;
	}
	return sequence;
}

static int
ssl_read_header(mem_access* memory){
	uint16_t version;
	uint16_t hdr = 0;

	hdr = memory->read() << 8;
	hdr |= memory->read();

	if(hdr == 0x3081){
		memory->read();
	}
	else if(hdr == 0x3082){
		memory->read();
		memory->read();
	}
	else
		return 0;
	version = memory->read() << 8;
	version |= memory->read();
	if(version != 0x0201)
		return 0;
	if(memory->read() != 0x00)
		return 0;
	if(!memory->error())
		return 1;
}

private_key*
ssl_find_private_key(mem_access* memory){
	int state = START;
	private_key* key = malloc(sizeof(private_key));

	while(!memory->error()){
		unsigned int startaddr = memory->tell();
		if(ssl_read_header(memory)){
			key->modulus = ssl_read_sequence(memory);
			key->E = ssl_read_sequence(memory);
			key->D = ssl_read_sequence(memory);
			key->P = ssl_read_sequence(memory);
			key->Q = ssl_read_sequence(memory);
			key->DP = ssl_read_sequence(memory);
			key->DQ = ssl_read_sequence(memory);
			key->IQ = ssl_read_sequence(memory);

			if(key->modulus &&
			   key->E &&
			   key->D &&
			   key->P &&
			   key->Q &&
			   key->DP &&
			   key->DQ &&
			   key->IQ)
				return key;
		}
		if(startaddr+1 != 0)
			memory->seek(startaddr+1);
	}
	return NULL;
}

#ifdef TEST

uint8_t data[] = 

	"\x30\x82\x02\x5d\x02\x01\x00\x02\x81\x81\x00\xd3\xc8\xf2\x92\x07"
    "\xd7\x53\x8a\x98\xe8\xd2\x6e\xfb\x11\xa6\x63\x98\x15\xaf\x34\xc7"
    "\x5d\x0e\x7a\xf3\x6b\xb8\x8e\x90\xa8\xa6\x94\x7d\xd9\x48\x5f\x9b"
    "\x52\xba\x39\x45\x7d\x8e\xcc\x4b\x90\x55\x51\x97\x76\xc1\x84\x0e"
    "\x14\x9a\x83\x5d\x2a\x91\x62\xd8\x23\xef\x27\x96\x22\x1a\x60\x00"
    "\xb9\x9b\x63\xb7\x92\xc7\xe4\xc8\xac\x65\x01\xd2\x4e\xe1\x14\x3f"
    "\xcd\xe5\x92\x75\x76\x84\x9e\x0d\x3d\xe0\x84\x8d\xa6\xbd\x91\xf7"
    "\x9b\x63\x4a\x44\x81\x74\x8e\xfd\x28\x49\xfa\xb4\x2e\xa7\x5d\xc8"
    "\x60\x8b\x90\x82\x23\xc7\x1b\x3e\x4b\x82\x9f\x02\x03\x01\x00\x01"
    "\x02\x81\x80\x7f\x8e\x18\x0b\x3c\x0d\xc4\x93\x45\x65\x74\xd8\xc2"
    "\xfe\xee\x68\x2d\xc1\x61\xb5\x38\x1e\x24\x0a\x89\x71\xe7\xc6\x75"
    "\xfa\x1d\xda\x71\xb1\xdc\x95\x38\xc7\xaa\xc1\xb3\xc8\xdb\xa8\xd0"
    "\x1a\x2e\x48\xe6\x20\xf0\x8c\xc8\xae\x40\x21\xbe\x8b\x54\xe5\x63"
    "\xa8\xc0\xf0\xce\x4c\xe8\xeb\xe1\xba\x95\x1a\x6f\x3b\x15\xbc\xd6"
    "\x42\x79\xdd\x71\xe9\x14\x9d\xb1\xa2\x09\xb2\x88\xa9\x6c\xc1\x62"
    "\x47\x21\xfc\xfe\x3d\x9e\x52\x60\xf9\x5e\x22\x4d\x02\xd1\x31\x42"
    "\xb4\x7c\x66\x15\x59\xe4\xad\x1b\x17\xfc\xc0\xa6\x42\x54\x95\xd9"
    "\xea\xe6\x01\x02\x41\x00\xeb\x14\x15\x0b\x21\x16\xe6\xfd\xc7\x44"
    "\x8c\xc3\x07\xcc\x56\xd0\x77\x31\x18\xb6\x1d\x21\x6b\xb0\x0a\xeb"
    "\xdd\xf3\x3b\x88\xf3\x47\xe6\x04\xc7\xc3\x08\xe7\x1c\x2d\xf0\x6c"
    "\xd9\xb5\x75\x3c\xbf\xcb\x54\xbc\x61\xe5\xba\x22\xdf\x17\x7b\x30"
    "\x14\x2c\x61\x4e\xe8\x0f\x02\x41\x00\xe6\xa2\x28\x3e\xb3\x98\x63"
    "\x1d\x2b\x00\xe3\xe9\xf3\x84\xf1\xdd\x39\xde\x21\x26\x93\xfd\xf7"
    "\x35\x6a\xab\xed\x0e\x8c\x55\x53\xbc\x15\x25\x0b\x2f\x56\xd0\x18"
    "\x06\xfb\xbd\x3e\x0d\x99\x1b\xcc\x6c\xe5\x9e\xee\x5c\x59\xc0\x94"
    "\x8c\xb9\x50\x16\x65\x2b\xa3\xac\x71\x02\x41\x00\xaa\x35\x54\x37"
    "\x46\xb2\x2a\x76\x3d\xbd\xe8\x9d\x9e\x1b\xda\x0f\xfe\xfd\x51\x83"
    "\x2a\xfc\x73\xd8\x77\x4f\xe8\xd8\xa0\x96\xbc\x6f\xa6\xa0\x77\x73"
    "\x44\x12\xf2\x58\x60\xe8\xaf\x89\xaf\x65\x59\x09\x84\x45\x12\x2d"
    "\x1f\xe4\xf7\xdb\x2c\x9a\xd7\x40\x7b\x04\x83\x09\x02\x41\x00\xc5"
    "\x27\x03\x4e\x2e\x4a\x6b\x17\xa9\x06\x36\xce\x6c\x0c\x4e\x91\x35"
    "\x81\x8c\x47\x25\x25\x69\xa8\xb1\x0a\x4d\x38\x2d\x80\x57\x77\x25"
    "\x80\x28\x22\x8c\x8c\x90\xad\x8f\x8f\x03\x37\xae\x95\x93\xd7\xd7"
    "\x26\x58\x4e\x7b\xae\xe3\x8d\xb4\x70\x20\xe4\xd5\x3c\x32\x11\x02"
    "\x40\x4d\x58\x4f\x6d\x60\x7e\x87\xe9\x0e\xd1\x22\xf6\xd5\x9d\x54"
    "\x07\x7a\x02\xfc\x47\xc2\x6b\x99\xc2\xe1\xb2\x07\xb0\x47\xf4\xd1"
    "\xc5\x8f\x7f\x72\xbe\xbd\x67\x90\x98\xa9\xe0\x40\x96\x18\xca\xe7"
    "\x1c\xa8\x0a\x08\xdc\x86\xcd\x59\xf5\x3b\x36\x42\x67\xac\x38\xfd"
    "\xd4";

int error = 0;
uint8_t *ptr = &(data[0]);
int my_read(void){
	if(ptr < &(data[0]) || ptr > &(data[0]) + sizeof(data)){
		error = 1;
	}
	if(error)
		return -1;
	ptr++;
	return *(ptr-1);
}

int my_seek(unsigned int addr){
	ptr = (void*)addr;
}

int my_tell(void){
	return (int)ptr;
}
int my_error(void){
	return error;
}

void printhex(uint8_t* data, size_t len){
	size_t i;
	for(i=0; i<len; ++i){
		printf("%x",data[i]);
	}
}

int main(void)
{
	mem_access memory;
	memory.read = my_read;
	memory.seek = my_seek;
	memory.tell = my_tell;
	memory.error = my_error;
/*	if(ssl_read_header(&memory)){
		printf("WIN\n");
	}*/
	private_key* key;
	if(key = ssl_find_private_key(&memory)){
		printf("EPIC WIN\n");
		printhex(key->modulus->data, key->modulus->len);
		printf("\n");
		printhex(key->E->data, key->E->len);
		printf("\n");
		printhex(key->D->data, key->D->len);
		printf("\n");
		printhex(key->P->data, key->P->len);
		printf("\n");
		printhex(key->Q->data, key->Q->len);
		printf("\n");
		printhex(key->DP->data, key->DP->len);
		printf("\n");
		printhex(key->DQ->data, key->DQ->len);
		printf("\n");
		printhex(key->IQ->data, key->IQ->len);
		printf("\n");
	}
    return 0;
}

#endif
